/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.model;
import android.content.ContentProviderOperation;
import android.content.ContentProviderResult;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.content.OperationApplicationException;
import android.net.Uri;
import android.os.RemoteException;
import android.util.Log;

import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.CallNote;
import net.micode.notes.data.Notes.DataColumns;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.data.Notes.TextNote;

import java.util.ArrayList;


public class Note {
    private ContentValues mNoteDiffValues;// 存储笔记变更的内容值
    private NoteData mNoteData;// 笔记数据
    private static final String TAG = "Note";// 日志标签
    /**
     * Create a new note id for adding a new note to databases为将新笔记添加到数据库中创建一个新的笔记ID
     */
    public static synchronized long getNewNoteId(Context context, long folderId) {//定义一个公共的静态同步方法，用于获取新的笔记ID。该方法接受两个参数：Context对象和folderId。
        // Create a new note in the database在数据库中创建一个新的笔记
        ContentValues values = new ContentValues();//创建一个新的ContentValues对象，用于存储要插入数据库的值。
        long createdTime = System.currentTimeMillis();//获取当前系统时间的毫秒表示。
        values.put(NoteColumns.CREATED_DATE, createdTime);//将创建时间添加到ContentValues对象中。
        values.put(NoteColumns.MODIFIED_DATE, createdTime);//将修改时间添加到ContentValues对象中，此时与创建时间相同。
        values.put(NoteColumns.TYPE, Notes.TYPE_NOTE);//将笔记类型添加到ContentValues对象中
        values.put(NoteColumns.LOCAL_MODIFIED, 1);//将本地修改标志添加到ContentValues对象中，设置为1。
        values.put(NoteColumns.PARENT_ID, folderId);//将父文件夹ID添加到ContentValues对象中。

        Uri uri = context.getContentResolver().insert(Notes.CONTENT_NOTE_URI, values);//通过ContentResolver的insert方法将ContentValues对象中的值插入到笔记数据库中，并返回新笔记的URI。

        long noteId = 0;//初始化一个long类型的变量noteId，用于存储新笔记的ID
        try {//开始一个try块，用于捕获异常
            noteId = Long.valueOf(uri.getPathSegments().get(1));//从URI中获取新笔记的ID。
        } catch (NumberFormatException e) {//捕获NumberFormatException类型的异常
            Log.e(TAG, "Get note id error :" + e.toString());//如果捕获到异常，则记录错误信息到日志中。
            noteId = 0;//将noteId设置为0，表示获取ID失败
        }
        if (noteId == -1) {
            throw new IllegalStateException("Wrong note id:" + noteId);//如果noteId为-1，则抛出一个IllegalStateException，表示获取到错误的笔记ID。
        }
        return noteId;//返回获取到的新笔记ID
    }

    public Note() {
        mNoteDiffValues = new ContentValues();//创建新的ContentValues对象
        mNoteData = new NoteData();//创建新的笔记数据
    }

    public void setNoteValue(String key, String value) {//定义了一个公共的方法setNoteValue，该方法用于设置笔记的数值。它接受两个参数：key和value。
        mNoteDiffValues.put(key, value);//将键值对(key, value)添加到mNoteDiffValues中。
        mNoteDiffValues.put(NoteColumns.LOCAL_MODIFIED, 1);//将本地修改标志设置为1，表示笔记已被修改。
        mNoteDiffValues.put(NoteColumns.MODIFIED_DATE, System.currentTimeMillis());//将修改日期设置为当前系统时间的毫秒表示。
    }

    public void setTextData(String key, String value) {//定义了一个公共的方法setTextData，用于设置笔记的文本数据。它接受两个参数：key和value。
        mNoteData.setTextData(key, value);}//调用mNoteData对象的setTextData方法，将键值对(key, value)添加到笔记数据中

    public void setTextDataId(long id) {//定义了一个公共的方法setTextDataId，用于设置笔记文本数据的ID。它接受一个参数：id。
        mNoteData.setTextDataId(id);//调用mNoteData对象的setTextDataId方法，设置笔记文本数据的ID。
    }

    public long getTextDataId() {
        return mNoteData.mTextDataId;//返回mNoteData对象中的文本数据ID
    }//定义了一个公共的方法getTextDataId，用于获取笔记文本数据的ID

    public void setCallDataId(long id) {
        mNoteData.setCallDataId(id);//调用mNoteData对象的setCallDataId方法，设置笔记通话数据的ID。
    }//定义了一个公共的方法setCallDataId，用于设置笔记通话数据的ID。它接受一个参数：id。

    public void setCallData(String key, String value) {
        mNoteData.setCallData(key, value);//调用mNoteData对象的setCallData方法，将键值对(key, value)添加到笔记通话数据中。
    }//定义了一个公共的方法setCallData，用于设置笔记的通话数据。它接受两个参数：key和value。


    public boolean isLocalModified() {//定义了一个公共的方法isLocalModified，用于检查笔记是否被本地修改。
        return mNoteDiffValues.size() > 0 || mNoteData.isLocalModified();//返回一个布尔值，表示mNoteDiffValues的大小是否大于0，或者mNoteData是否被本地修改。
    }

    public boolean syncNote(Context context, long noteId) {//定义了一个公共的方法syncNote，用于同步笔记。
        if (noteId <= 0) {
            throw new IllegalArgumentException("Wrong note id:" + noteId);//如果笔记ID小于等于0，则抛出一个IllegalArgumentException，表示获取到错误的笔记ID。
        }

        if (!isLocalModified()) {//检查笔记是否被本地修改
            return true;//如果笔记没有被本地修改，则返回true
        }

        /**
         * In theory, once data changed, the note should be updated on {@link NoteColumns#LOCAL_MODIFIED} and
         * {@link NoteColumns#MODIFIED_DATE}. For data safety, though update note fails, we also update the
         * note data info
         * 理论上，一旦数据发生变化，笔记应该在{@link NoteColumns#LOCAL_MODIFIED}和{@link NoteColumns#MODIFIED_DATE}上更新。
         * 为了数据安全，即使更新笔记失败，我们也会更新笔记数据信息
         */
        if (context.getContentResolver().update(
                ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, noteId), mNoteDiffValues, null,
                null) == 0) {
            Log.e(TAG, "Update note error, should not happen");
            // Do not return, fall through不要返回，继续执行
        }
        mNoteDiffValues.clear();

        if (mNoteData.isLocalModified()
                && (mNoteData.pushIntoContentResolver(context, noteId) == null)) {
            return false;
        }

        return true;
    }

    private class NoteData {
        private long mTextDataId;// 文本数据ID

        private ContentValues mTextDataValues;// 文本数据内容值

        private long mCallDataId;// 通话数据ID

        private ContentValues mCallDataValues; // 通话数据内容值

        private static final String TAG = "NoteData"; // 日志标签

        /**
         * 这是一个构造函数。
         *在这个构造函数中，它初始化了两个成员变量 `mTextDataValues` 和 `mCallDataValues`，它们都是用来存储键值对的对象
         *同时，它还初始化了两个 ID 变量 `mTextDataId` 和 `mCallDataId`。
         */
        public NoteData() {
            mTextDataValues = new ContentValues();
            mCallDataValues = new ContentValues();
            mTextDataId = 0;
            mCallDataId = 0;
        }

        /**
         * 这是一个用来检查是否有本地修改的方法。
         * 它通过检查 `mTextDataValues` 和 `mCallDataValues` 的大小来判断是否有本地修改。
         * @return
         */
        boolean isLocalModified() {
            return mTextDataValues.size() > 0 || mCallDataValues.size() > 0;
        }

        /**
         * 这是一个设置文本数据 ID 的方法。
         * 如果传入的 ID 小于等于 0，将会抛出一个异常。
         * @param id
         */
        void setTextDataId(long id) {
            if(id <= 0) {
                throw new IllegalArgumentException("Text data id should larger than 0");
            }
            mTextDataId = id;
        }

        /**
         * 这是一个设置通话数据 ID 的方法。
         * 如果传入的 ID 小于等于 0，将会抛出一个异常。
         * @param id
         */
        void setCallDataId(long id) {
            if (id <= 0) {
                throw new IllegalArgumentException("Call data id should larger than 0");
            }
            mCallDataId = id;
        }

        /**
         * 这是一个设置通话数据的方法。
         * 它将键值对存储到 `mCallDataValues` 中，并且更新了一些其他的笔记差异值
         * @param key
         * @param value
         */
        void setCallData(String key, String value) {
            mCallDataValues.put(key, value);
            mNoteDiffValues.put(NoteColumns.LOCAL_MODIFIED, 1);
            mNoteDiffValues.put(NoteColumns.MODIFIED_DATE, System.currentTimeMillis());
        }

        /**
         * 这是一个设置文本数据的方法。
         * 它将键值对存储到 `mTextDataValues` 中，并且更新了一些其他的笔记差异值。
         * @param key
         * @param value
         */
        void setTextData(String key, String value) {
            mTextDataValues.put(key, value);
            mNoteDiffValues.put(NoteColumns.LOCAL_MODIFIED, 1);
            mNoteDiffValues.put(NoteColumns.MODIFIED_DATE, System.currentTimeMillis());
        }

        Uri pushIntoContentResolver(Context context, long noteId) {
            /**
             * Check for safety
             */
            if (noteId <= 0) {// 检查 noteId 是否合法
                throw new IllegalArgumentException("Wrong note id:" + noteId);
            }

            ArrayList<ContentProviderOperation> operationList = new ArrayList<ContentProviderOperation>();// 创建一个操作列表
            ContentProviderOperation.Builder builder = null;

            if(mTextDataValues.size() > 0) {// 如果有文本数据
                mTextDataValues.put(DataColumns.NOTE_ID, noteId);// 设置笔记 ID
                if (mTextDataId == 0) {// 如果文本数据 ID 为 0，表示需要插入新数据
                    mTextDataValues.put(DataColumns.MIME_TYPE, TextNote.CONTENT_ITEM_TYPE);
                    Uri uri = context.getContentResolver().insert(Notes.CONTENT_DATA_URI,
                            mTextDataValues);
                    try {
                        setTextDataId(Long.valueOf(uri.getPathSegments().get(1)));
                    } catch (NumberFormatException e) {
                        Log.e(TAG, "Insert new text data fail with noteId" + noteId);
                        mTextDataValues.clear();
                        return null;
                    }
                } else {// 否则，表示需要更新数据
                    builder = ContentProviderOperation.newUpdate(ContentUris.withAppendedId(
                            Notes.CONTENT_DATA_URI, mTextDataId));
                    builder.withValues(mTextDataValues);
                    operationList.add(builder.build());
                }
                mTextDataValues.clear();
            }

            if(mCallDataValues.size() > 0) { // 如果有通话数据
                mCallDataValues.put(DataColumns.NOTE_ID, noteId); // 设置笔记 ID
                if (mCallDataId == 0) {// 如果通话数据 ID 为 0，表示需要插入新数据
                    mCallDataValues.put(DataColumns.MIME_TYPE, CallNote.CONTENT_ITEM_TYPE);
                    Uri uri = context.getContentResolver().insert(Notes.CONTENT_DATA_URI,
                            mCallDataValues);
                    try {
                        setCallDataId(Long.valueOf(uri.getPathSegments().get(1)));
                    } catch (NumberFormatException e) {
                        Log.e(TAG, "Insert new call data fail with noteId" + noteId);
                        mCallDataValues.clear();
                        return null;
                    }
                } else {// 否则，表示需要更新数据
                    builder = ContentProviderOperation.newUpdate(ContentUris.withAppendedId(
                            Notes.CONTENT_DATA_URI, mCallDataId));
                    builder.withValues(mCallDataValues);
                    operationList.add(builder.build());
                }
                mCallDataValues.clear();
            }

            if (operationList.size() > 0) {
                try {// 应用操作列表到内容提供程序
                    ContentProviderResult[] results = context.getContentResolver().applyBatch(
                            Notes.AUTHORITY, operationList);
                    return (results == null || results.length == 0 || results[0] == null) ? null
                            : ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, noteId);
                } catch (RemoteException e) {
                    Log.e(TAG, String.format("%s: %s", e.toString(), e.getMessage()));
                    return null;
                } catch (OperationApplicationException e) {
                    Log.e(TAG, String.format("%s: %s", e.toString(), e.getMessage()));
                    return null;
                }
            }
            return null;
        }
    }
}
/**
 * 这个类定义了一个用于管理笔记数据的对象，
 */