/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * 导入包和声明包名和版权声明：
 *    - 代码中导入了一些需要使用的类，以及声明了包名和版权信息。
 */
package net.micode.notes.model;

import android.appwidget.AppWidgetManager;
import android.content.ContentUris;
import android.content.Context;
import android.database.Cursor;
import android.text.TextUtils;
import android.util.Log;

import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.CallNote;
import net.micode.notes.data.Notes.DataColumns;
import net.micode.notes.data.Notes.DataConstants;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.data.Notes.TextNote;
import net.micode.notes.tool.ResourceParser.NoteBgResources;


/**
 * 这段代码是一个工作笔记的类，用于管理笔记数据。
 */
public class WorkingNote {//`WorkingNote` 类用于管理笔记数据，包含了一些成员变量和方法。
    // Note for the working note
    private Note mNote;//用于存储笔记对象
    // Note Id
    private long mNoteId;//笔记的 ID
    // Note content
    private String mContent;//笔记的内容
    // Note mode
    private int mMode;//笔记的模式

    private long mAlertDate;//提醒日期

    private long mModifiedDate;//最后修改日期

    private int mBgColorId;//背景颜色的 ID

    private int mWidgetId;//小部件的 ID

    private int mWidgetType;//小部件的类型

    private long mFolderId;//文件夹的 ID

    private Context mContext;//上下文对象

    private static final String TAG = "WorkingNote";

    private boolean mIsDeleted;//笔记是否被删除

    private NoteSettingChangedListener mNoteSettingStatusListener;//笔记设置状态的监听器

    /**
     * 数据查询相关的常量和数组：
     * 定义了查询数据和笔记的投影（projection）和相关的列索引常量。
     */
    public static final String[] DATA_PROJECTION = new String[] {
            DataColumns.ID,
            DataColumns.CONTENT,
            DataColumns.MIME_TYPE,
            DataColumns.DATA1,
            DataColumns.DATA2,
            DataColumns.DATA3,
            DataColumns.DATA4,
    };

    public static final String[] NOTE_PROJECTION = new String[] {
            NoteColumns.PARENT_ID,
            NoteColumns.ALERTED_DATE,
            NoteColumns.BG_COLOR_ID,
            NoteColumns.WIDGET_ID,
            NoteColumns.WIDGET_TYPE,
            NoteColumns.MODIFIED_DATE
    };

    private static final int DATA_ID_COLUMN = 0;

    private static final int DATA_CONTENT_COLUMN = 1;

    private static final int DATA_MIME_TYPE_COLUMN = 2;

    private static final int DATA_MODE_COLUMN = 3;

    private static final int NOTE_PARENT_ID_COLUMN = 0;

    private static final int NOTE_ALERTED_DATE_COLUMN = 1;

    private static final int NOTE_BG_COLOR_ID_COLUMN = 2;

    private static final int NOTE_WIDGET_ID_COLUMN = 3;

    private static final int NOTE_WIDGET_TYPE_COLUMN = 4;

    private static final int NOTE_MODIFIED_DATE_COLUMN = 5;

    // New note construct
    private WorkingNote(Context context, long folderId) {
        mContext = context;
        mAlertDate = 0;
        mModifiedDate = System.currentTimeMillis();
        mFolderId = folderId;
        mNote = new Note();
        mNoteId = 0;
        mIsDeleted = false;
        mMode = 0;
        mWidgetType = Notes.TYPE_WIDGET_INVALIDE;
    }

    // Existing note construct

    /**
     * 构造方法：
     *    - 有两个私有构造方法，一个是用于创建新笔记的，一个是用于加载现有笔记的
     * @param context
     * @param noteId
     * @param folderId
     */
    private WorkingNote(Context context, long noteId, long folderId) {
        mContext = context;
        mNoteId = noteId;
        mFolderId = folderId;
        mIsDeleted = false;
        mNote = new Note();
        loadNote();
    }

    /**
     * `loadNote` 方法：
     *  从数据库加载笔记的相关信息，
     *  包括文件夹 ID、背景颜色、小部件 ID、小部件类型、提醒日期和最后修改日期。
     */
    private void loadNote() {
        Cursor cursor = mContext.getContentResolver().query(
                ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, mNoteId), NOTE_PROJECTION, null,
                null, null);

        if (cursor != null) {
            if (cursor.moveToFirst()) {
                mFolderId = cursor.getLong(NOTE_PARENT_ID_COLUMN);
                mBgColorId = cursor.getInt(NOTE_BG_COLOR_ID_COLUMN);
                mWidgetId = cursor.getInt(NOTE_WIDGET_ID_COLUMN);
                mWidgetType = cursor.getInt(NOTE_WIDGET_TYPE_COLUMN);
                mAlertDate = cursor.getLong(NOTE_ALERTED_DATE_COLUMN);
                mModifiedDate = cursor.getLong(NOTE_MODIFIED_DATE_COLUMN);
            }
            cursor.close();
        } else {
            Log.e(TAG, "No note with id:" + mNoteId);
            throw new IllegalArgumentException("Unable to find note with id " + mNoteId);
        }
        loadNoteData();
    }

    /**
     * `loadNoteData` 方法：
     *  从数据库加载笔记的具体数据，包括内容、模式等
     */
    private void loadNoteData() {
        Cursor cursor = mContext.getContentResolver().query(Notes.CONTENT_DATA_URI, DATA_PROJECTION,
                DataColumns.NOTE_ID + "=?", new String[] {
                    String.valueOf(mNoteId)
                }, null);

        if (cursor != null) {
            if (cursor.moveToFirst()) {
                do {
                    String type = cursor.getString(DATA_MIME_TYPE_COLUMN);
                    if (DataConstants.NOTE.equals(type)) {
                        mContent = cursor.getString(DATA_CONTENT_COLUMN);
                        mMode = cursor.getInt(DATA_MODE_COLUMN);
                        mNote.setTextDataId(cursor.getLong(DATA_ID_COLUMN));
                    } else if (DataConstants.CALL_NOTE.equals(type)) {
                        mNote.setCallDataId(cursor.getLong(DATA_ID_COLUMN));
                    } else {
                        Log.d(TAG, "Wrong note type with type:" + type);
                    }
                } while (cursor.moveToNext());
            }
            cursor.close();
        } else {
            Log.e(TAG, "No data with id:" + mNoteId);
            throw new IllegalArgumentException("Unable to find note's data with id " + mNoteId);
        }
    }

    public static WorkingNote createEmptyNote(Context context, long folderId, int widgetId,//创建新笔记的私有构造方法
            int widgetType, int defaultBgColorId) {
        WorkingNote note = new WorkingNote(context, folderId);
        note.setBgColorId(defaultBgColorId);
        note.setWidgetId(widgetId);
        note.setWidgetType(widgetType);
        return note;
    }


    public static WorkingNote load(Context context, long id) {//这是一个静态方法，用于加载现有笔记。它返回一个WorkingNote对象。
        return new WorkingNote(context, id, 0);
    }

    /**
     * 下面这个方法用于保存笔记。
     * 它首先调用`isWorthSaving()`方法来判断是否值得保存，然后根据情况执行保存操作。
     * 如果存在与该笔记相关的小部件，并且设置状态监听器不为空，则调用小部件变更的回调方法。
     * @return
     */
    public synchronized boolean saveNote() {
        if (isWorthSaving()) {
            if (!existInDatabase()) {
                if ((mNoteId = Note.getNewNoteId(mContext, mFolderId)) == 0) {
                    Log.e(TAG, "Create new note fail with id:" + mNoteId);
                    return false;
                }
            }

            mNote.syncNote(mContext, mNoteId);

            /**
             * Update widget content if there exist any widget of this note
             */
            if (mWidgetId != AppWidgetManager.INVALID_APPWIDGET_ID
                    && mWidgetType != Notes.TYPE_WIDGET_INVALIDE
                    && mNoteSettingStatusListener != null) {
                mNoteSettingStatusListener.onWidgetChanged();
            }
            return true;
        } else {
            return false;
        }
    }

    public boolean existInDatabase() {//这个方法用于判断笔记是否存在于数据库中
        return mNoteId > 0;
    }

    private boolean isWorthSaving() {//这个方法用于判断笔记是否值得保存
        if (mIsDeleted || (!existInDatabase() && TextUtils.isEmpty(mContent))
                || (existInDatabase() && !mNote.isLocalModified())) {
            return false;
        } else {
            return true;
        }
    }

    public void setOnSettingStatusChangedListener(NoteSettingChangedListener l) {//这个方法用于设置设置状态变更监听器。
        mNoteSettingStatusListener = l;
    }

    public void setAlertDate(long date, boolean set) {//这个方法用于设置提醒日期，并在设置状态监听器不为空的情况下调用提醒日期变更的回调方法
        if (date != mAlertDate) {
            mAlertDate = date;
            mNote.setNoteValue(NoteColumns.ALERTED_DATE, String.valueOf(mAlertDate));
        }
        if (mNoteSettingStatusListener != null) {
            mNoteSettingStatusListener.onClockAlertChanged(date, set);
        }
    }

    public void markDeleted(boolean mark) {//这个方法用于标记笔记是否被删除，并在存在与该笔记相关的小部件并且设置状态监听器不为空的情况下调用小部件变更的回调方法
        mIsDeleted = mark;
        if (mWidgetId != AppWidgetManager.INVALID_APPWIDGET_ID
                && mWidgetType != Notes.TYPE_WIDGET_INVALIDE && mNoteSettingStatusListener != null) {
                mNoteSettingStatusListener.onWidgetChanged();
        }
    }

    public void setBgColorId(int id) {//这个方法用于设置背景颜色的ID，并在设置状态监听器不为空的情况下调用背景颜色变更的回调方法。

        if (id != mBgColorId) {
            mBgColorId = id;
            if (mNoteSettingStatusListener != null) {
                mNoteSettingStatusListener.onBackgroundColorChanged();
            }
            mNote.setNoteValue(NoteColumns.BG_COLOR_ID, String.valueOf(id));
        }
    }

    public void setCheckListMode(int mode) {//这个方法用于设置清单模式，并在设置状态监听器不为空的情况下调用清单模式变更的回调方法。

        if (mMode != mode) {
            if (mNoteSettingStatusListener != null) {
                mNoteSettingStatusListener.onCheckListModeChanged(mMode, mode);
            }
            mMode = mode;
            mNote.setTextData(TextNote.MODE, String.valueOf(mMode));
        }
    }

    public void setWidgetType(int type) {//这个方法用于设置小部件类型
        if (type != mWidgetType) {
            mWidgetType = type;
            mNote.setNoteValue(NoteColumns.WIDGET_TYPE, String.valueOf(mWidgetType));
        }
    }

    public void setWidgetId(int id) {//这个方法用于设置小部件ID
        if (id != mWidgetId) {
            mWidgetId = id;
            mNote.setNoteValue(NoteColumns.WIDGET_ID, String.valueOf(mWidgetId));
        }
    }

    public void setWorkingText(String text) {//这个方法用于设置工作文本
        if (!TextUtils.equals(mContent, text)) {
            mContent = text;
            mNote.setTextData(DataColumns.CONTENT, mContent);
        }
    }

    public void convertToCallNote(String phoneNumber, long callDate) {//这个方法用于将笔记转换为通话记录
        mNote.setCallData(CallNote.CALL_DATE, String.valueOf(callDate));
        mNote.setCallData(CallNote.PHONE_NUMBER, phoneNumber);
        mNote.setNoteValue(NoteColumns.PARENT_ID, String.valueOf(Notes.ID_CALL_RECORD_FOLDER));
    }

    public boolean hasClockAlert() {//这个方法用于判断笔记是否有闹钟提醒
        return (mAlertDate > 0 ? true : false);
    }//这个方法用于判断笔记是否有闹钟提醒。


    public String getContent() {//这个方法用于获取笔记内容
        return mContent;
    }//这个方法用于获取笔记内容。

    public long getAlertDate() {//这个方法用于获取提醒日期
        return mAlertDate;
    }//这个方法用于获取提醒日期

    public long getModifiedDate() {//这个方法用于获取笔记修改日期
        return mModifiedDate;
    }//这个方法用于获取笔记修改日期

    public int getBgColorResId() {//这个方法用于获取背景颜色的资源ID
        return NoteBgResources.getNoteBgResource(mBgColorId);
    }

    public int getBgColorId() {//这个方法用于获取背景颜色的ID
        return mBgColorId;
    }//这个方法用于获取背景颜色的ID。

    public int getTitleBgResId() {//这个方法用于获取标题背景的资源ID
        return NoteBgResources.getNoteTitleBgResource(mBgColorId);}  //这个方法用于获取标题背景的资源ID。

    public int getCheckListMode() {//这个方法用于获取清单模式
        return mMode;
    }//这个方法用于获取清单模式。

    public long getNoteId() {//这个方法用于获取笔记ID
        return mNoteId;
    } //这个方法用于获取笔记ID

    public long getFolderId() {//这个方法用于获取笔记所属文件夹的ID
        return mFolderId;
    } //这个方法用于获取笔记所属文件夹的ID。

    public int getWidgetId() {//这个方法用于获取小部件的ID
        return mWidgetId;
    } //这个方法用于获取小部件的ID。

    public int getWidgetType() {//这个方法用于获取小部件的类型
        return mWidgetType;
    }

    /**
     * 下面这是一个接口，定义了一系列方法，用于监听笔记设置的变化。
     * 具体而言，包括背景颜色变化、闹钟提醒设置变化、通过小部件创建笔记以及切换清单模式和普通模式时的状态变化。
     */
    public interface NoteSettingChangedListener {
        /**
         * Called when the background color of current note has just changed
         */
        void onBackgroundColorChanged();

        /**
         * Called when user set clock
         */
        void onClockAlertChanged(long date, boolean set);

        /**
         * Call when user create note from widget
         */
        void onWidgetChanged();

        /**
         * Call when switch between check list mode and normal mode
         * @param oldMode is previous mode before change
         * @param newMode is new mode
         */
        void onCheckListModeChanged(int oldMode, int newMode);
    }
}
